/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.rel

import com.persagy.server.services.base.RService
import com.persagy.server.datacenter.models.entities.rel.REqInSh
import com.persagy.server.datacenter.models.entities.rel.REqInShList
import com.persagy.server.datacenter.models.entities.rel.RShInEqList
import com.persagy.base.extensions.toJson
import com.persagy.mybatis.SMybatisDao
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse
import org.json.JSONObject

/**
 * 设备所在竖井关系服务
 *
 * @author 付国宝
 */
object REqInShService: RService<REqInSh>(SMybatisDao(REqInSh::class.java)) {

    /**
     * 创建设备所在竖井关系
     *
     * @param    rEqInSh     设备所在竖井关系对象
     */
    fun create(rEqInSh: REqInSh): SCreateResponse<REqInSh>{
        return try {
            val projectId = SPageContext.getHeader("projectId")
            rEqInSh.projectId = projectId
            val isInsert = insert(rEqInSh)
            if (isInsert){
                val jsonObject = JSONObject()
                jsonObject.put("type", "11")
                jsonObject.put("projectId", projectId)
                jsonObject.put("id", rEqInSh.equipId)
                jsonObject.put("obj_type", "EquipinShaft")
                jsonObject.put("linkId",rEqInSh.shaftId)

                /** 发送消息 */
                messageHandler.sendMessage(jsonObject.toString())
//                kafkaService.sendNotice("datacenterrel", RelationNotice(projectId!!, "EquipinShaft", "11", IdUtils.uuidCreate(),  rEqInSh.equipId!!,
//                        rEqInSh.shaftId!!),false)


                SCreateResponse(SResponseType.success)
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function create()

    /**
     * 根据对象删除
     *
     * @param     rEqInSh      关系对象实例
     * @return
     */
    fun deletes(rEqInSh: REqInSh): SBaseResponse{
        return try {
            val projectId = SPageContext.getHeader("projectId")
            val isDelete = delete(rEqInSh)
            /** 删除成功 */
            if (isDelete) {
                val jsonObject = JSONObject()
                jsonObject.put("type", "12")
                jsonObject.put("projectId", projectId)
                jsonObject.put("id", rEqInSh.equipId)
                jsonObject.put("obj_type", "EquipinShaft")
                jsonObject.put("linkId",rEqInSh.shaftId)

                /** 发送消息 */
                messageHandler.sendMessage(jsonObject.toString())
//                kafkaService.sendNotice("datacenterrel", RelationNotice(projectId!!, "EquipinShaft", "12", IdUtils.uuidCreate(),  rEqInSh.equipId!!,
//                        rEqInSh.shaftId!!),false)

                val sBaseResponse = SBaseResponse(SResponseType.success)
                sBaseResponse.message = rEqInSh.toJson()
                sBaseResponse
            } else {
                SBaseResponse(SResponseType.failure)
            }
        }catch (e: Exception){
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure, e.message!!)
        }
    } // Function deletes()

    /**
     * 创建设备所在竖井关系
     *
     * @param    rEqInShList     设备和竖井关系的对象列表
     */
    fun createEq(rEqInShList: REqInShList): SCreateResponse<REqInSh>{
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            delete("equipID = '${rEqInShList.equipId}'")
            val projectId = SPageContext.getHeader("projectId")
            for (shaftId in rEqInShList.shaftIdList!!){
                val rEqInSh = REqInSh()
                rEqInSh.equipId = rEqInShList.equipId
                rEqInSh.shaftId = shaftId
                rEqInSh.projectId = projectId
                val replace = replace(rEqInSh)
                if (replace){
                    successList.add(rEqInSh)
                }else{
                    failureList.add(rEqInSh)
                }
            }
            if (successList.size>0){
                val jsonObject = JSONObject()
                jsonObject.put("type", "13")
                jsonObject.put("projectId", projectId)
                jsonObject.put("id", rEqInShList.equipId)
                jsonObject.put("obj_type", "EquipinShaft")
                jsonObject.put("linkId",successList.toJson())
                /** 发送消息 */
                messageHandler.sendMessage(jsonObject.toString())
//                kafkaService.sendNotice("datacenterrel", RelationNotice(projectId!!, "EquipinShaft", "13", IdUtils.uuidCreate(),  rEqInShList.equipId!!,null),false)
                val sCreateResponse = SCreateResponse<REqInSh>(SResponseType.success)
                if (failureList.size>0){
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                }
                sCreateResponse
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createEq()

    /**
     * 创建所在竖井的设备关系
     *
     * @param    rShInEqList     设备和竖井关系的对象列表
     */
    fun createsSh(rShInEqList: RShInEqList): SCreateResponse<REqInSh>{
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            delete("shaftID = '${rShInEqList.shaftId}'")
            val projectId = SPageContext.getHeader("projectId")
            for (equipId in rShInEqList.equipIdList!!){
                val rEqInSh = REqInSh()
                rEqInSh.shaftId = rShInEqList.shaftId
                rEqInSh.equipId = equipId
                rEqInSh.projectId = projectId
                val replace = replace(rEqInSh)
                if (replace){
                    successList.add(rEqInSh)
                }else{
                    failureList.add(rEqInSh)
                }
            }
            if (successList.size>0){
                val jsonObject = JSONObject()
                jsonObject.put("type", "13")
                jsonObject.put("projectId", projectId)
                jsonObject.put("id", rShInEqList.shaftId)
                jsonObject.put("obj_type", "EquipinShaft")
                jsonObject.put("linkId",successList.toJson())
                /** 发送消息 */
                messageHandler.sendMessage(jsonObject.toString())
//                kafkaService.sendNotice("datacenterrel", RelationNotice(projectId!!, "EquipinShaft", "13", IdUtils.uuidCreate(),  rShInEqList.shaftId!!,null),false)
                if (failureList.size>0){
                    val sCreateResponse = SCreateResponse<REqInSh>(SResponseType.success)
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                    sCreateResponse
                }else{
                    val sCreateResponse = SCreateResponse<REqInSh>(SResponseType.success)
                    sCreateResponse
                }
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e:Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createsSh()

    /**
     * 创建所在竖井的设备关系
     *
     * @param    rShInEqList     设备和竖井关系的对象列表
     */
    fun createsShEq(rShInEqList: RShInEqList): SCreateResponse<REqInSh>{
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
//            delete("shaftID = '${rShInEqList.shaftId}'")
            val projectId = SPageContext.getHeader("projectId")
            for (equipId in rShInEqList.equipIdList!!){
                val rEqInSh = REqInSh()
                rEqInSh.shaftId = rShInEqList.shaftId
                rEqInSh.equipId = equipId
                rEqInSh.projectId = projectId
                val replace = replace(rEqInSh)
                if (replace){
                    successList.add(rEqInSh)
                }else{
                    failureList.add(rEqInSh)
                }
            }
            if (successList.size>0){
                val jsonObject = JSONObject()
                jsonObject.put("type", "14")
                jsonObject.put("projectId", projectId)
                jsonObject.put("id", rShInEqList.shaftId)
                jsonObject.put("obj_type", "EquipinShaft")
                jsonObject.put("linkId",successList.toJson())

                /** 发送消息 */
                messageHandler.sendMessage(jsonObject.toString())
//                kafkaService.sendNotice("datacenterrel", RelationNotice(projectId!!, "EquipinShaft", "14", IdUtils.uuidCreate(),  rShInEqList.shaftId!!,null),false)
                if (failureList.size>0){
                    val sCreateResponse = SCreateResponse<REqInSh>(SResponseType.success)
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                    sCreateResponse
                }else{
                    val sCreateResponse = SCreateResponse<REqInSh>(SResponseType.success)
                    sCreateResponse
                }
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e:Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createsSh()

} // Object REqInShService