/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.syn

import com.alibaba.fastjson.JSONArray
import com.alibaba.fastjson.JSONObject

/**
 * 配置文件
 *
 * @author wx  <zhangweixin@sagacloud.com>
 * @date  2020/11/1 21:45
 */
object Configure {

    /** 数据中台消息之创建 */
    const val RWDCREATE = "instanceObjectCreate"
    /** 数据中台消息之修改 */
    const val RWDUPDATE = "instanceObjectUpdate"
    /** 数据中台消息之删除 */
    const val RWDDELETE = "instanceObjectDelete"

    /** 对象类型 - 项目 */
    const val PROJECT = "project"
    /** 对象类型 - 建筑 */
    const val BUILDING = "building"
    /** 对象类型 - 楼层 */
    const val FLOOR = "floor"
    /** 对象类型 - 设备 */
    const val EQUIPMENT = "equipment"
    /** 对象类型 - 空间 */
    const val SPACE = "space"
    /** 对象类型 - 竖井 */
    const val SHAFT = "shaft"
    /** 对象类型 - 系统 */
    const val SYSTEM = "system"
    /** 对象类型 - 部件 */
    const val COMPONENT = "component"

    /** 创建 */
    const val CREATE = "create"
    /** 修改 */
    const val UPDATE = "update"
    /** 删除 */
    const val DELETE = "delete"


    /**
     * 创建关系列表
     *
     * @param graphCode     图类型编码
     * @param relCode       边类型编码
     * @param objFrom       主对象 id
     * @param objTo         从对象 id
     */
    fun toJsonObjArray(graphCode: String, relCode: String, objFrom: String, objTo: String): JSONArray {
        /** 列表 */
        val jsonArray = JSONArray()
        /** 对象 */
        val jsonObject = JSONObject()
        /** 图类型编码 */
        jsonObject["graphCode"] = graphCode
        /** 边类型编码 */
        jsonObject["relCode"] = relCode
        /** 主对象 id */
        jsonObject["objFrom"] = objFrom
        /** 从对象 id */
        jsonObject["objTo"] = objTo
        /** 添加到列表 */
        jsonArray.plus(jsonObject)
        return jsonArray
    }

    /**
     * 创建关系列表
     *
     * @param graphCode     图类型编码
     * @param relCode       边类型编码
     * @param objFrom       主对象 id
     * @param objTo         从对象 id
     */
    fun toJsonObj(graphCode: String, relCode: String, objFrom: String, objTo: String?): JSONObject {
        /** 对象 */
        val jsonObject1 = JSONObject()
        /** 对象 */
        val jsonObject = JSONObject()
        /** 图类型编码 */
        jsonObject["graphCode"] = graphCode
        /** 边类型编码 */
        jsonObject["relCode"] = relCode
        /** 主对象 id */
        jsonObject["objFrom"] = objFrom
        if (!objTo.isNullOrEmpty()){
            /** 从对象 id */
            jsonObject["objTo"] = objTo
        }
        /** 添加到对象 */
        jsonObject1["criteria"] = jsonObject
        return jsonObject1
    }

}