/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.scheduler.controllers

import com.alibaba.fastjson.JSON
import com.persagy.base.extensions.toJson
import com.persagy.server.scheduler.models.DownloadEntity
import com.persagy.server.scheduler.models.DownloadSpeedRequest
import com.persagy.server.scheduler.models.TaskEntity
import com.persagy.server.scheduler.service.BaseDataService
import com.persagy.server.scheduler.service.JMSProducer
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.requests.SUpdateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.annotations.Api
import io.swagger.annotations.ApiOperation
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController
import java.util.*

@Api(tags = ["download接口"])
@RestController
@RequestMapping("/download")
open class DownloadController {
    companion object {
        // 日志
        private val logger = LoggerFactory.getLogger(DownloadController::class.java)
    } // Companion object

    @Autowired
    lateinit var activeMQ : JMSProducer
    /**
     * 下载速度
     */
    @ApiOperation(value = "下载速度", notes = "")
    @PostMapping(value = ["/speed"])
    fun speed(@RequestBody request: DownloadSpeedRequest): SBaseResponse {
        logger.debug("task =${request.toJson()}")
        // 如果已下载完, 则给完成时间加上
        if(request.totalBytes != null && request.totalBytes!! > -1L && request.taskId != null) {
            if (request.downloadedBytes == request.totalBytes) {
                val task = TaskEntity()
                task.id = request.taskId
                task.downloadFinishTime = Date()
                BaseDataService.taskService.update(task, arrayListOf("downloadFinishTime"))
            }
        }
        // 发送到MQ
        try {
            activeMQ.sendMessage(JSON.toJSONString(request))
        }catch (ignore : Exception){}
        return SBaseResponse()
    } // Function create()

//    /**
//     * 创建Task
//     */
//    @ApiOperation(value = "创建Task", notes = "")
//    @PostMapping(value = ["/create"])
//    fun create(@RequestBody taskEntity: ArrayList<DownloadEntity>): SCreateResponse<DownloadEntity> {
//        logger.debug("equip =${taskEntity.toJson()}")
//        val request = SCreateRequest<DownloadEntity>()
//        request.content = taskEntity
//        return downloadService.createList(request)
//    } // Function create()
    /**
     * 创建Task
     */
    @ApiOperation(value = "创建Task", notes = "")
    @PostMapping(value = ["/create"])
    fun create(@RequestBody request: SCreateRequest<DownloadEntity>): SCreateResponse<DownloadEntity> {
        logger.debug("task =${request.toJson()}")
        return BaseDataService.downloadService.createList(request)
    } // Function create()
    /**
     * 更新Task
     *
     * @param   request     更新请求
     * @return  更新结果应答信息
     */
    @ApiOperation(value="更新Task", notes="")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: SUpdateRequest<DownloadEntity>): SBaseResponse {
        return BaseDataService.downloadService.updateList(request)
    } // Function update()

    /**
     * 查询Task
     *
     * @param   request     查询请求
     * @return  查询结果应答信息
     */
    @ApiOperation(value="查询Task", notes="")
    @PostMapping(value = ["/query"])
    fun query(@RequestBody request: SQueryRequest): SQueryResponse<DownloadEntity> {
        return BaseDataService.downloadService.pageQuery(request)
    } // Function query()


    /**
     * 删除下载Task
     *
     * @param   idList      TaskEntity ID列表
     * @return  删除结果应答信息
     */
    @ApiOperation(value = "删除Task", notes = "")
    @PostMapping(value = ["/delete"])
    fun delete(@RequestBody idList: ArrayList<DownloadEntity>): SBaseResponse {
        return BaseDataService.downloadService.deleteByKeysList(idList)
    } // Function delete()
}